using System;
using System.Drawing.Design;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using RepPP;

namespace RepPP.Toolkit {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a WEB control and
    /// supports atom property.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class AtmControlUtil : ControlUtil {
	    /// <summary>Common interface for all controls attached to a Rep++ Atom</summary>
	    public interface IAtmControl : IRepControl {
	        /// <summary>Rep++ application object</summary>
	        RepPP.Application           Application { get; }
	        /// <summary>Field name</summary>
	        AtmFullName                 AtomFullName {  get; set; }
	        /// <summary>Trigger an event to indicate a component property has changed</summary>
	        void                        TriggerComponentChange();
	    }
	    
	    /// <summary>Atom text attached to the control</summary>
	    protected string                m_strAtomText;
	    /// <summary>Full name of the atom</summary>
	    protected AtmFullName           m_atmFullName;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public AtmControlUtil(object parent, bool bResDispenserOnly) : base(parent, bResDispenserOnly) {
		    m_strAtomText   = null;
		    m_atmFullName   = new AtmFullName();
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPInitialized:           Called when the Rep++ is initialized
		/// </summary>
        /// <param name="sender">       Sender object</param>
        /// <param name="e">            Event arguments</param>
        //
        //********************************************************************************
        public override void RepPPInitialized(object sender, InitNotificationEventArgs e) {
            switch(e.Code) {
            case InitNotificationCode.sdThisProject:
            case InitNotificationCode.sdSameConnection:
                m_strAtomText = null;
                break;
            }
            base.RepPPInitialized(sender, e);
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPResetting:             Called when the Rep++ is reset. Called only in design mode.
		/// </summary>
        //
        //********************************************************************************
        public override void RepPPResetting(object sender, InitNotificationEventArgs e) {
            switch(e.Code) {
            case InitNotificationCode.sdThisProject:
            case InitNotificationCode.sdSameConnection:
                m_strAtomText = "<" + AtomFullName.ToString() + ">";
                break;
            default:
                break;
            }
            base.RepPPResetting(sender, e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// AtomFullName:          Full name of the field attached to the control
        /// </summary>
        //
        //********************************************************************************
		public AtmFullName AtomFullName {
		    get {
		        return(m_atmFullName);
		    }
		    set {
		        m_atmFullName   = value;
		        m_strAtomText   = null;
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// AtomText:               Atom text attached to the control
		/// </summary>
        //
        //********************************************************************************
		public string AtomText {
		    get {
		        RepPP.Application   app;
		        
		        if (m_strAtomText == null) {
		            app = Application;
                    if (app == null || m_atmFullName.AtomName == String.Empty) {
                        m_strAtomText = "<" + m_atmFullName.AtomName + ">";
                    } else if (!app.IsAtomExist(m_atmFullName.AtomName, m_atmFullName.AtomType)) {
                        m_strAtomText = "<?" + m_atmFullName + "?>";
                    } else {
		                m_strAtomText = app.GetAtomText(m_atmFullName.AtomName, m_atmFullName.AtomType);
		            }
		        }
		        return(m_strAtomText);
		    }
		}

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the AtomFullName property
	    /// </summary>
        //
        //********************************************************************************
        public class AtmControlEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.Modal;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                FormAtmControlPropPage          frmAtmControlPropPage;
                AtmControlUtil.IAtmControl      atmControl;
                IWindowsFormsEditorService      edSvc;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    if (edSvc != null) {
                        atmControl            = context.Instance as AtmControlUtil.IAtmControl;
                        frmAtmControlPropPage = new FormAtmControlPropPage(atmControl);
                        try {
                            edSvc.ShowDialog(frmAtmControlPropPage);
                            objRetVal = atmControl.AtomFullName;
                        } finally {
                            frmAtmControlPropPage.Dispose();
                        }
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class AtmControlEditor

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the Atom type property
	    /// </summary>
        //
        //********************************************************************************
        public class AtmTypeEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                AtmFullName                     atmFullName;
                IWindowsFormsEditorService      edSvc;
                RepPP.Application               app;
                FldControlUtil.PopupListBox     listBox;
                AtomType                        eType;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    app   = RepPP.Application.GetApplicationForDesign(provider);
                    if (edSvc != null && app != null) {
                        atmFullName             = context.Instance as AtmFullName;
                        objRetVal               = atmFullName.AtomType;
                        listBox                 = new FldControlUtil.PopupListBox(edSvc);
                        listBox.Location        = new System.Drawing.Point(0, 0);
                        listBox.Name            = "listBox";
                        listBox.Size            = new System.Drawing.Size(168, 186);
                        listBox.TabIndex        = 1;
                        listBox.BorderStyle     = System.Windows.Forms.BorderStyle.None;
                        listBox.Items.Add(AtomType.sdAtomMsg.ToString());
                        listBox.Items.Add(AtomType.sdAtomErr.ToString());
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            eType      = (AtomType)Enum.Parse(typeof(AtomType), (listBox.SelectedItem as string));
                            objRetVal  = eType;
                            if (eType != atmFullName.AtomType) {
                                atmFullName.AtomName = "";
                            }
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class AtomTypeEditor

        //********************************************************************************
        //
	    /// <summary>
	    /// Implement an editor for the Atom name property
	    /// </summary>
        //
        //********************************************************************************
        public class AtmNameEditor : System.Drawing.Design.UITypeEditor {

            //********************************************************************************
            //
            /// <summary>
            /// GetEditStyle:           Gets the style of edition
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Type of editor (modal)
            /// </returns>
            //
            //********************************************************************************
            public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context) {
                UITypeEditorEditStyle eRetVal;
                
                if (context != null && context.Instance != null) {
                    eRetVal = UITypeEditorEditStyle.DropDown;
                } else {
                    eRetVal = base.GetEditStyle(context);
                }
                return(eRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// EditValue:              Edit a value
            /// </summary>
            /// <param name="context">  Context</param>
            /// <param name="provider"> Provider</param>
            /// <param name="value">    Value to edit</param>
            /// <returns>
            /// New value
            /// </returns>
            //
            //********************************************************************************
            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value) {
                object                          objRetVal = null;
                AtmFullName                     atmFullName;
                IWindowsFormsEditorService      edSvc;
                RepPP.Application               app;
                FldControlUtil.PopupListBox     listBox;
                string                          strAtomList;

                if (context != null && context.Instance != null && provider != null) {
                    edSvc = provider.GetService(typeof(IWindowsFormsEditorService)) as IWindowsFormsEditorService;
                    app   = RepPP.Application.GetApplicationForDesign(provider);
                    if (edSvc != null && app != null) {
                        atmFullName             = context.Instance as AtmFullName;
                        objRetVal               = atmFullName.AtomName;
                        listBox                 = new FldControlUtil.PopupListBox(edSvc);
                        listBox.Location        = new System.Drawing.Point(0, 0);
                        listBox.Name            = "listBox";
                        listBox.Size            = new System.Drawing.Size(168, 186);
                        listBox.TabIndex        = 1;
                        listBox.BorderStyle     = System.Windows.Forms.BorderStyle.None;
                        strAtomList             = app.GetAtomList(atmFullName.AtomType);
                        listBox.Items.AddRange(strAtomList.Split(','));
                        edSvc.DropDownControl(listBox);
                        if (listBox.SelectedItem != null) {
                            objRetVal = listBox.SelectedItem as string;
                        }   
                    }
                }
                if (objRetVal == null) {
                    objRetVal = base.EditValue(context, provider, value);
                }
                return (objRetVal);
            }
        } // Class AtmNameEditor        
    } // Class AtmControlUtil
} // Namespace