using System;
using System.Collections;
using System.ComponentModel;
using System.Web.UI;
using System.ComponentModel.Design;
using System.Reflection;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// Represents a clickable item within a context menu.
    /// </summary>
    /// <remarks>
    /// Items can be handled either within the client or on
    /// the server via a postback based on whether or not the
    /// JavaScriptCommand property is declared
    /// </remarks>
    //
    //********************************************************************************
    [TypeConverter(typeof(ExpandableObjectConverter))]
	public class ContextMenuItem
	{
        ///<summary>The item's unique Id</summary>
        private string m_strId;
        ///<summary>The item's display text</summary>
        private string m_strText;
        ///<summary>The server-side command argument</summary>
        private string m_strCommandName;
        ///<summary>The client-side script</summary>
        private string m_strJavaScriptCommand;
        ///<summary>The item's tooltip</summary>
        private string m_strTooltip;
        ///<summary>The url for the item's icon image</summary>
        private string m_strImageURL;
        ///<summary>Indicates if the item will be displayed or not</summary>
        private bool m_bVisible = true;

        //********************************************************************************
        //
        /// <summary>
        /// ContextMenuItem:         Class Constructor
        /// </summary>
        //
        //********************************************************************************
        public ContextMenuItem(){
		}

        //********************************************************************************
        //
        /// <summary>
        /// ContextMenuItem:                                Class Constructor
        /// </summary>
        /// <param name="strId">                            Unique id</param>
        /// <param name="strText">                          Text to display</param>
        /// <param name="strCommandName">                   Server-side command name</param>
        /// <param name="strJavaScriptCommand">             Client-side script</param>
        /// <param name="strTooltip">                       Tooltip</param>
        //
        //********************************************************************************
        public ContextMenuItem(string strId,
                               string strText, 
                               string strCommandName,
                               string strJavaScriptCommand,
                               string strTooltip) {
            m_strId                 = strId;
			m_strText               = strText;
			m_strCommandName        = strCommandName;
            m_strJavaScriptCommand  = strJavaScriptCommand;
            m_strTooltip            = strTooltip;
		}

        //********************************************************************************
        //
        /// <summary>
        /// Text:     Gets or sets the unique id for this item.
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [DefaultValue("")]
        [Description("Unique id for this menu item")]
        [NotifyParentProperty(true)]
        public string Id {
            get {
                return(m_strId);
            }
            set {
                m_strId = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Text:     Gets or sets the text to display for this item.
        /// </summary>
        //
        //********************************************************************************
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("Text of the menu item")]
        [NotifyParentProperty(true)]
		public string Text{
			get {
                return(m_strText);
            }
			set {
                m_strText = value;
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// ImageURL:     Gets or sets the url of the image to display for this item.
        /// </summary>
        //
        //********************************************************************************
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("Url of the image of the menu item")]
        [NotifyParentProperty(true)]
        public string ImageURL {
            get {
                return(m_strImageURL);
            }
            set {
                m_strImageURL = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Visible:     Gets or sets the visiblity for this item.
        /// </summary>
        //
        //********************************************************************************
        [Category("Appearance")]
        [DefaultValue(true)]
        [Description("Indicates if the item will be displayed or not")]
        [NotifyParentProperty(true)]
        public bool Visible {
            get {
                return(m_bVisible);
            }
            set {
                m_bVisible = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// CommandName:        Gets or Sets the command name to associate with 
        ///                     this menu item
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [DefaultValue("")]
        [Description("Command name to associate with the menu item")]
        [NotifyParentProperty(true)]
		public string CommandName{
			get {
                return(m_strCommandName);
            }
			set {
                m_strCommandName = value;
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Tooltip:            The tooltip associated with this menu item
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [DefaultValue("")]
        [Description("The tooltip for the menu item")]
        [NotifyParentProperty(true)]
        public string Tooltip{
			get {
                return(m_strTooltip);
            }
			set {
                m_strTooltip = value;
            }
		}

        //********************************************************************************
        //
        /// <summary>
        /// JavaScriptCommand:              Gets or sets the client-side script to be executed
        ///                                 when this item is selected.
        /// </summary>
        //
        //********************************************************************************
        [Category("Behavior")]
        [DefaultValue("")]
        [Description("Javascript code to associate with the menu item")]
        public string JavaScriptCommand {
            get {
                return(m_strJavaScriptCommand);
            }
            set {
                m_strJavaScriptCommand = value;
            }
        }

	}//class ContextMenuItem


    //********************************************************************************
    //
    /// <summary>
    /// A collection of ContextMenuItem objects in a ContextMenu control
    /// </summary>
    //
    //********************************************************************************
    public class ContextMenuItemCollection : CollectionBase {
        //********************************************************************************
        //
        /// <summary>
        /// ContextMenuItemCollection:         Class Constructor
        /// </summary>
        //
        //********************************************************************************
		public ContextMenuItemCollection() {
		}
	
        //********************************************************************************
        //
        /// <summary>
        /// this[]:                 Gets and sets the element at the specified position
        /// </summary>
        /// <param name="index">    The name of the element to be get or set</param>
        //
        //********************************************************************************
		public ContextMenuItem this[int index] {
			get { 
                return((ContextMenuItem)InnerList[index]); 
            }
			set {
                InnerList[index] = value;
            }
		}
		
        //********************************************************************************
        //
        /// <summary>
        /// Add:                    Adds an object to the end of the collection
        /// </summary>
        /// <param name="item">     The ContextMenuItem to be added</param>
        //
        //********************************************************************************
		public void Add(ContextMenuItem item){
			InnerList.Add(item);
		}

        //********************************************************************************
        //
        /// <summary>
        /// AddAt:                  Adds an object at the specified position in the collection
        /// </summary>
        /// <param name="index">    The Index where the item will be added</param>
        /// <param name="item">     The ContextMenuItem to be added</param>
        //
        //********************************************************************************
		public void AddAt(int index, ContextMenuItem item){
			InnerList.Insert(index, item);
		}

	}//class ContextMenuItemCollection


}//namespace RepPP.Toolkit.Web
