using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Xml;
using System.Drawing;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// ListViewGInst: Create a list view around a group instance
	/// </summary>
	[ToolboxItem(true)]
	[ToolboxBitmap(typeof(ListViewGInst),"SCListView.bmp")]
	public class ListViewGInst : ListViewBase, ISelectionBufferUI {
        /// <summary>
        /// Rep++ Group instance attached to the listview
        /// </summary>
        protected RepPP.GroupInstance           m_ginst;
        /// <summary>
        /// Field which must be autorenum after a move
        /// </summary>
        protected RepPP.Field                   m_fldAutoRenum;
        /// <summary>
        /// Increment for the auto-renum
        /// </summary>
        protected int                           m_iAutoInc;

        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
        public ListViewGInst() {
            m_ginst         = null;
            m_fldAutoRenum  = null;
            m_iAutoInc      = 10;
        }

        //*********************************************************
        //
        /// <summary>
        /// Group:                      Group assign to the listview
        /// </summary>
        //
        //*********************************************************
		[Description("Group associates to the listview."), Browsable(false) ]
        public override RepPP.Group Group {
            get {
                return(base.Group);
            }
            set {
                if (m_grp != value) {
                    m_ginst = null;
                    base.Group = value;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GroupInstance:              Group instance assign to the listview
        /// </summary>
        //
        //*********************************************************
		[Description("Group instance associates to the listview."), Browsable(false) ]
        public virtual RepPP.GroupInstance GroupInstance {
            get {
                return(m_ginst);
            }
            set {
                m_ginst = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AutoRenumField:             Field use to auto-renum the line after a move
        /// </summary>
        //
        //*********************************************************
 	    [Description("Field use to auto-renum the line after a move."), Browsable(false) ]
        public RepPP.Field AutoRenumField {
            get {
                return(m_fldAutoRenum);
            }
            set {
                m_fldAutoRenum = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// AutoRenumIncrement:         Value use to auto-renum the lines after a move
        /// </summary>
        //
        //*********************************************************
		[Description("Value use to auto-renum the lines after a move."), Browsable(false) ]
        public int AutoRenumIncrement {
            get {
                return(m_iAutoInc);
            }
            set {
                m_iAutoInc = value;
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// CreateListViewItem:         Creates a new listview item. Can be override to creates derived class
        /// </summary>
        /// <param name="iLineIndex">   Line index</param>
        /// <returns>
        /// New listview item
        /// </returns>
        //
        //*********************************************************
        protected override ListViewItem CreateListViewItem(int iLineIndex) {
            return(new ListViewLineItem(m_ginst, iLineIndex));
        }

        //*********************************************************
        //
        /// <summary>
        /// Fill:                           Fill the listview with the value of the group instance
        /// </summary>
        /// <param name="ginst">            Group instance to use to fill the listview</param>
        /// <param name="bUseDesc">         true to use choice description instead of choice code</param>
        //
        //*********************************************************
        public virtual void Fill(RepPP.GroupInstance ginst, bool bUseDesc) {
            if (m_grp == null) {
                Group = ginst.Group;
            }
            m_ginst = ginst;
            InternalFill(m_ginst, bUseDesc, true);
        }

        //*********************************************************
        //
        /// <summary>
        /// Fill:                           Fill the listview with the value of the group instance
        /// </summary>
        /// <param name="ginst">            Group instance to use to fill the listview</param>
        /// <param name="bUseDesc">         true to use choice description instead of choice code</param>
        /// <param name="bDispDelLine">     True to display deleted lines in the listview</param>
        //
        //*********************************************************
        public virtual void Fill(RepPP.GroupInstance ginst, bool bUseDesc, bool bDispDelLine) {
            if (m_grp == null) {
                Group = ginst.Group;
            }
            m_ginst = ginst;
            InternalFill(m_ginst, bUseDesc, bDispDelLine);
        }

        //*********************************************************
        //
        /// <summary>
        /// FindItemFromLineBookmark:   Find an item using the line bookmark
        /// </summary>
        /// <param name="iLineBmk">     Line bookmark</param>
        /// <returns>
        /// item or null if not found
        /// </returns>
        //
        //*********************************************************
        public virtual ListViewLineItem FindItemFromLineBookmark(int iLineBmk) {
            ListViewLineItem    itemRetVal = null;
            
            foreach (ListViewItem item in Items) {
                if (item is ListViewLineItem) {
                    itemRetVal = item as ListViewLineItem;
                    if (itemRetVal.LineBookmark == iLineBmk) {
                        break;
                    } else {
                        itemRetVal = null;
                    }
                }
            }
            return(itemRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// GetItemKey:                 Gets the currently selected item unique key
        /// </summary>
        /// <returns>
        /// Item key or null if none
        /// </returns>
        //
        //*********************************************************
        public string GetItemKey() {
            string  strRetVal;
            
            if (m_ginst == null || m_ginst.CurrentLine == -1) {
                strRetVal = null;
            } else {
                strRetVal = "";
                foreach (RepPP.Field fld in m_ginst.Group.Fields) {
                    if (fld.IsPrimaryKey) {
                        if (strRetVal.Length != 0) {
                            strRetVal += "\0";
                        }
                        strRetVal += fld.Value;
                    }
                }
            }
            return(strRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// FindItemFromKey:            Find the item from the specified key
        /// </summary>
        /// <param name="strItemKey">   Item key</param>
        /// <returns>
        /// Item related to the key or null if not found
        /// </returns>
        //
        //*********************************************************
        public virtual ListViewLineItem FindItemFromKey(string strItemKey) {
            ListViewLineItem    itemRetVal = null;
            int                 iCount;
            int                 iPos;
            string[]            arrKey;
            object[]            arrObj;
            
            if (strItemKey != null && strItemKey.Length != 0) {
                arrKey = strItemKey.Split('\0');
                iCount = arrKey.Length;
                arrObj = new object[iCount * 2];
                iPos   = 0;
                foreach (RepPP.Field fld in m_ginst.Group.Fields) {
                    if (fld.IsPrimaryKey) {
                        if (iPos == iCount) {
                            throw new System.ApplicationException("Key item overflow!");
                        }
                        arrObj[iPos*2]      = fld;
                        arrObj[iPos*2+1]    = arrKey[iPos];
                        iPos++;
                    }
                }
                if (iPos != iCount) {
                    throw new System.ApplicationException("Key item mismatch");
                }
                if (m_ginst.FindValue(false, arrObj) == 0) {
                    itemRetVal = FindItemFromLineBookmark(m_ginst.LineBookmark(m_ginst.CurrentLine));
                }
            }
            return(itemRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// SelectItemFromKey:          Select the item from the specified key
        /// </summary>
        /// <param name="strItemKey">   Item key</param>
        /// <returns>
        /// true if a selection occurs, false if not.
        /// </returns>
        //
        //*********************************************************
        public virtual bool SelectItemFromKey(string strItemKey) {
            bool                bRetVal;
            ListViewLineItem    item;
            
            item = FindItemFromKey(strItemKey);
            if (item != null) {
                SelectedItem  = item;
                bRetVal       = true;
            } else {
                bRetVal       = false;
            }
            return(bRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// InsertItem:             Insert an item in the list
        /// </summary>
        /// <param name="iIndex">   Index where to add it in the list. -1 to append</param>
        /// <param name="bUseDesc"> true to use choice description instead of choice code</param>
        /// <returns>
        /// New item inserted
        /// </returns>
        //
        //*********************************************************
        public virtual ListViewItem InsertItem(int iIndex, bool bUseDesc) {
            return(InternalInsertItem(iIndex, m_ginst, bUseDesc));
        }

        //*********************************************************
        //
        /// <summary>
        /// UpdateItem:             Update an item in the list
        /// </summary>
        /// <param name="iIndex">   Index of the item to change</param>
        //
        //*********************************************************
        public virtual void UpdateItem(int iIndex) {
            InternalUpdateItem(iIndex, m_ginst);
        }

        //*********************************************************
        //
        /// <summary>
        /// DeleteItem:             Delete an item from the list
        /// </summary>
        /// <param name="iIndex">   Index of the item to update</param>
        //
        //*********************************************************
        public void DeleteItem(int iIndex) {
            InternalDeleteItem(iIndex);
        }

        //******************************************************************
        //
        /// <summary>
        /// AutoRenum:                  Autorenum the lines after a move
        /// </summary>
        //
        //******************************************************************
        protected virtual void AutoRenum()
        {
            int                 iCurBmk;
            int                 iCurLine;
            int                 iLineBmk;
            RepPP.LineState     eLineState;
            int                 iVal;
            string              strVal;
            ListViewLineItem    gitem;
            
            if (m_fldAutoRenum != null && m_ginst != null && m_ginst.LineCount != 0) {
                iCurBmk = m_ginst.LineBookmark(m_ginst.CurrentLine);
                iVal    = m_iAutoInc;
                // Be careful, line count can change because of changes in PK
                for (int iIndex = 0; iIndex < m_ginst.LineCount; iIndex++) {
                    if (m_ginst.ThisLineState(iIndex) != RepPP.LineState.sdLineDeleted) {
                        m_ginst.SelectLine(iIndex, false, false);
                        iLineBmk = m_ginst.LineBookmark(iIndex);
                        gitem    = FindItemFromLineBookmark(iLineBmk);
                        strVal = iVal.ToString();
                        if (m_fldAutoRenum.Value != strVal) {
                            m_fldAutoRenum.Value = strVal;
                            eLineState         = m_ginst.LineState; // Force the state change of the line.
                            iLineBmk           = m_ginst.LineBookmark(iIndex);
                            gitem.LineBookmark = iLineBmk;
                            RefreshItemInSelectionBuffer(m_ginst);
                        }
                        iVal += m_iAutoInc;
                    }
                }
                iCurLine = m_ginst.LineFromBookmark(iCurBmk);
                if (iCurLine != -1 && iCurLine != m_ginst.CurrentLine) {
                    m_ginst.SelectLine(iCurLine, false, false);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// MoveItem:                   Move an item after the specified position
        /// </summary>
        /// <param name="item">         Item to move</param>
        /// <param name="itemAfter">    Item after which the item is move or null for the first pos</param>
        //
        //******************************************************************
        public override void MoveItem(ListViewItem item, ListViewItem itemAfter)
        {
            int                 iItemLine;
            int                 iItemAfterLine;
            ListViewLineItem    gitem;
            ListViewLineItem    gitemAfter;

            base.MoveItem(item, itemAfter);
            if (item != itemAfter) {
                gitem       = item as ListViewLineItem;
                gitemAfter  = itemAfter as ListViewLineItem;
                if (gitem != null) {
                    iItemLine       = m_ginst.LineFromBookmark(gitem.LineBookmark);
                    iItemAfterLine  = (gitemAfter == null) ? -1 : m_ginst.LineFromBookmark(gitemAfter.LineBookmark);
                    if (m_ginst.CurrentLine != iItemLine) {
                        m_ginst.SelectLine(iItemLine);
                    }
                    if (iItemLine != (iItemAfterLine + 1)) {
                        m_ginst.MoveLineTo(iItemAfterLine);
                        AutoRenum();
                    }
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnAfterSelect:              Called after selecting an item
        /// </summary>
        /// <param name="e">            Event argument</param>
        //
        //*********************************************************
        protected override void OnAfterSelect(AfterSelectEventArgs e) {
            ListViewLineItem                item;
            int                             iLinePos;
            
            if (SelectedItems.Count != 0) {
                if (SelectedItems[0] is ListViewLineItem) {
                    item     = SelectedItems[0] as ListViewLineItem;
                    iLinePos = m_ginst.LineFromBookmark(item.LineBookmark);
                    if (iLinePos != -1) {
                        if (iLinePos != m_ginst.CurrentLine) {
                            m_ginst.SelectLine(iLinePos, true, true);
                        }
                    }
                }
            }
            base.OnAfterSelect(e);
        }

        //******************************************************************
        //
        /// <summary>
        /// AddItemToSelectionBuffer:   Add a new item to the selection buffer
        /// </summary>
        /// <param name="ginstSB">      Group instance of the selection buffer</param>
        /// <param name="bDuplicate">   true to duplicate, false to insert new one</param>
        //
        //******************************************************************
        public virtual void AddItemToSelectionBuffer(RepPP.GroupInstance ginstSB, bool bDuplicate)  // implement ISelectionBufferUI
        { 
            ListViewItem    item;

            item          = InsertItem(-1, true);
            SelectedItem  = item;
            item.EnsureVisible();
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshItemInSelectionBuffer:   Refresh an item
        /// </summary>
        /// <param name="ginstData">        Group instance use to refresh the data</param>
        //
        //******************************************************************
        public virtual void RefreshItemInSelectionBuffer(RepPP.GroupInstance ginstData) // implement ISelectionBufferUI
        {
            RepPP.GroupInstance ginst;
            int                 iLineBmk;
            ListViewLineItem    item;
            
            ginst    = GroupInstance;
            if (ginst.Group != ginstData.Group) {
                ginst.Group.CopyFieldsFrom(ginstData.Group, RepPP.CopyFieldOpt.sdCopyState);
            }
            iLineBmk = ginst.LineBookmark(ginst.CurrentLine);
            if (iLineBmk != -1) {
                item = FindItemFromLineBookmark(iLineBmk);
                if (item == null) {
                    item = SelectedItem as ListViewLineItem;
                }
                if (item != null) {
                    UpdateItem(item.Index);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// DeleteItemFromSelectionBuffer:  Delete an item from the selection buffer
        /// </summary>
        /// <param name="bReselect">        true to select another row</param>
        //
        //******************************************************************
        public virtual void DeleteItemFromSelectionBuffer(bool bReselect) // implement ISelectionBufferUI
        {
            ListViewItem        item;
            int                 iIndex;
            
            item   = (SelectedItem != null) ? SelectedItem : m_selectionInfo.m_itemCurrent;
            iIndex = item.Index;
            GroupInstance.DeleteLine();
            DeleteItem(iIndex);
            if (bReselect) {
                if (iIndex >= Items.Count) {
                    iIndex = Items.Count - 1;
                }
                if (iIndex != -1) {
                    SelectedItem = Items[iIndex];
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GetItemCount:                   Gets the number of item
        /// </summary>
        //
        //******************************************************************
        public virtual int GetItemCount()      // implement ISelectionBufferUI
        {
            return(Items.Count);
        }

        //******************************************************************
        //
        /// <summary>
        /// GetCurrentItem:                 Gets the current item
        /// </summary>
        //
        //******************************************************************
        public virtual int GetCurrentItem()    // implement ISelectionBufferUI
        {
            int     iRetVal;

            iRetVal = (SelectedItems.Count == 0) ? -1 : SelectedItems[0].Index;
            return(iRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// SelectItem:                     Select the specified item
        /// </summary>
        //
        //******************************************************************
        public virtual bool SelectItem(int iPos)   // implement ISelectionBufferUI
        {
            bool         bRetVal;
            ListViewItem item;

            if (iPos < 0) {
                bRetVal = false;
            } else if (iPos >= Items.Count) {
                bRetVal = false;
            } else {
                item         = Items[iPos];
                SelectedItem = item;
                item.EnsureVisible();
                bRetVal              = true;
            }
            return(bRetVal);
        }
	} // class
} // namespace
