using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using RepPP;
using System.Xml;
using RepPP.Toolkit.Window;

namespace RepPP.Toolkit.Window {

    //*********************************************************
    //
    /// <summary>
    /// Class handling New MDI form. There must be one instance of this class
    /// for each MDI list of top form. Each top form must register to the list
    /// using the RegisterForm method.
    /// </summary>
    //
    //*********************************************************
    public abstract class NMDIFormHelperBase : IDisposable {

        //*********************************************************
        //
        /// <summary>
        /// Interface support by form which are managed by the New MDI manager
        /// </summary>
        //
        //*********************************************************
        public interface INMDIForm {
            /// <summary>Ask before closing the form</summary>
            /// <returns>true if the form can be closed. false if not</returns>
            bool                AskBeforeClosing();
        }

        //*********************************************************
        //
        /// <summary>
        /// Class representing the top "active" form
        /// </summary>
        //
        //*********************************************************
        protected class TopForm {
            /// <summary>Form</summary>
            public Form         frmTop;
            /// <summary>Command Holder</summary>
            public CmdHolder    cmdHolder;
            /// <summary>Visibility Indicator</summary>
            public bool         bVisible;
        };
        
        /// <summary>Style of tiling which can be applied to the forms</summary>
        public enum ArrangeStyleE {
            /// <summary>Tiled verticaly</summary>
            Vertical    = 1,
            /// <summary>Tiled Horizontaly</summary>
            Horizontal  = 2,
            /// <summary>Tiled in cascade</summary>
            Cascade     = 3
        };
        /// <summary>Stores all opened forms</summary>
        protected   ArrayList                           m_arrTopForm;
        /// <summary>Active form</summary>
        protected   TopForm							    m_topFormActive	= null;
        /// <summary>Number of visible top form</summary>
        protected   int                                 m_iVisibleCount = 0;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor.
        /// </summary>
        //
        //******************************************************************
        public NMDIFormHelperBase() {
            m_arrTopForm    = new ArrayList();
        }

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                        Dispose the form
        /// </summary>
        //
        //******************************************************************
        public virtual void Dispose() {
        }

        //******************************************************************
        //
        /// <summary>
        /// FindTopForm:            Finds a top form in the list
        /// </summary>
        /// <param name="frm">      Form object to find</param>
        /// <param name="iIndex">   Returns the index of the found top form or -1 if not found</param>
        /// <returns>
        /// Top form or null if not found
        /// </returns>
        //
        //******************************************************************
        private TopForm FindTopForm(Form frm, out int iIndex) {
            TopForm topFormRetVal = null;
            
            iIndex = 0;
            foreach (TopForm topForm in m_arrTopForm) {
                if (topForm.frmTop == frm) {
                    topFormRetVal = topForm;
                    break;
                }
            }
            if (topFormRetVal == null) {
                iIndex = -1;
            }
            return(topFormRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// FindTopForm:            Finds a top form in the list
        /// </summary>
        /// <param name="frm">      Form object to find</param>
        /// <returns>
        /// Top form or null if not found
        /// </returns>
        //
        //******************************************************************
        private TopForm FindTopForm(Form frm) {
            int     iIndex;
            
            return(FindTopForm(frm, out iIndex));
        }

        //******************************************************************
        //
        /// <summary>
        /// FillWindowsMenu:            Initialize the windows menu
        /// </summary>
        /// <param name="cmdHolder">    Command holder</param>
        //
        //******************************************************************
        private void FillWindowsCmd(CmdHolder cmdHolder) {
            cmdHolder.AddCmd("WindowsMinimizeAll",
                new CmdHolder.OnCommand(OnWindowsMinimizeAllClick),        CmdHolder.CmdStateE.Disabled);
            cmdHolder.AddCmd("WindowsMaximizeAll",
                new CmdHolder.OnCommand(OnWindowsMaximizeAllClick),        CmdHolder.CmdStateE.Disabled);
            cmdHolder.AddCmd("WindowsRestoreAll",
                new CmdHolder.OnCommand(OnWindowsRestoreAllClick),         CmdHolder.CmdStateE.Disabled);
            cmdHolder.AddCmd("WindowsArrangeVertically",
                new CmdHolder.OnCommand(OnWindowsArrangeVerticallyClick),  CmdHolder.CmdStateE.Disabled);
            cmdHolder.AddCmd("WindowsArrangeHorizontally",
                new CmdHolder.OnCommand(OnWindowsArrangeHorizontallyClick),CmdHolder.CmdStateE.Disabled);
            cmdHolder.AddCmd("WindowsArrangeInCascade",
                new CmdHolder.OnCommand(OnWindowsArrangeInCascadeClick),   CmdHolder.CmdStateE.Disabled);
            cmdHolder.AddCmd("WindowsCloseAll",
                new CmdHolder.OnCommand(OnWindowsCloseAllClick),           CmdHolder.CmdStateE.Disabled);
        }

        //******************************************************************        
        //
        /// <summary>
        /// RegisterForm:                   Register a form
        /// </summary>
        /// <param name="frm">              Form to register</param>
        /// <param name="bVisible">         true to show form in the top form list</param>
        /// <param name="cmdHolder">        Command holder</param>
        //
        //******************************************************************        
        protected virtual void RegisterForm(Form frm, bool bVisible, CmdHolder cmdHolder) {
            TopForm topForm;
            
            if (frm is INMDIForm) {
                topForm           = new TopForm();
                topForm.cmdHolder = cmdHolder;
                topForm.frmTop    = frm;
                topForm.bVisible  = bVisible;
                m_arrTopForm.Add(topForm);
                frm.Closed     += new EventHandler(OnNMDIFormClosed);
                frm.Activated  += new EventHandler(OnNMDIFormActivated);
                m_topFormActive = topForm;
                if (bVisible) {
                    m_iVisibleCount++;
                }
            } else {
                throw new System.ApplicationException("The form must supports the INMDIForm interface");
            }
            FillWindowsCmd(cmdHolder);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnNMDIFormClosed:       Remove the form from the list when its closing
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        protected virtual void OnNMDIFormClosed(object sender, System.EventArgs e) {
            TopForm topForm;
            
            if (m_arrTopForm != null) {
                topForm = FindTopForm((Form)sender);
                if (topForm != null) {
                    if (topForm.bVisible) {
                        m_iVisibleCount--;
                    }
                    topForm.frmTop.Closed    -= new EventHandler(OnNMDIFormClosed);
                    topForm.frmTop.Activated -= new EventHandler(OnNMDIFormActivated);
                    m_arrTopForm.Remove(topForm);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnNMDIFormActivated:    Called when a form is activated so we can now which one is actives.
        /// </summary>
        /// <param name="sender">   Event sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        protected virtual void OnNMDIFormActivated(object sender, System.EventArgs e) {
            TopForm     topForm;
            
            topForm = FindTopForm((Form)sender);
            if (topForm != null) {
                m_topFormActive = topForm;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RefreshWindowsCmd:		    Adjust Windows menu depending on the current context.
        /// </summary>
        /// <param name="frmActive">    Currently active top form</param>
        /// <param name="cmdHolder">    Command holder</param>
        /// <returns>
        /// Returns the number of visible forms
        /// </returns>
        //
        //******************************************************************
        protected virtual void RefreshWindowsCmd(Form frmActive, CmdHolder cmdHolder) {
            cmdHolder.SetCmdState("WindowsMinimizeAll",
                (m_iVisibleCount != 0) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
            cmdHolder.SetCmdState("WindowsMaximizeAll",
                (m_iVisibleCount != 0) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
            cmdHolder.SetCmdState("WindowsRestoreAll",
                (m_iVisibleCount != 0) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
            cmdHolder.SetCmdState("WindowsArrangeVertically",
                (IsArrangeAvailable(ArrangeStyleE.Vertical)) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
            cmdHolder.SetCmdState("WindowsArrangeHorizontally",
                (IsArrangeAvailable(ArrangeStyleE.Horizontal)) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
            cmdHolder.SetCmdState("WindowsArrangeInCascade",
                (IsArrangeAvailable(ArrangeStyleE.Cascade)) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
            cmdHolder.SetCmdState("WindowsCloseAll",
                (m_iVisibleCount != 0) ? CmdHolder.CmdStateE.Enabled : CmdHolder.CmdStateE.Disabled);
        }

        //******************************************************************
        //
        /// <summary>
        /// FindNMDIForm:           Find a New MDI form in the array using its type. Can be use to see if a
        ///                         specific forms is already present before creating a new one.
        /// </summary>
        /// <param name="type">     Form type to search for</param>
        /// <returns>
        /// Top form or null if not found
        /// </returns>
        //
        //******************************************************************
        public Form FindNMDIForm(System.Type type) {
            Form    frmRetVal = null;
		    
            if (m_arrTopForm != null) {
                foreach(TopForm topForm in m_arrTopForm) {
                    if (topForm.frmTop.GetType() == type) {
                        frmRetVal = topForm.frmTop;
                        break;
                    }
                }
            }
            return(frmRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// NMDIFormListArray:      Gets the top form array
        /// </summary>
        //
        //******************************************************************
        public Form[] NMDIFormListArray {
            get {
                Form[]  arrFrm;
                int     iIndex = 0;
                
                arrFrm = new Form[m_arrTopForm.Count];
                foreach (TopForm topForm in m_arrTopForm) {
                    arrFrm[iIndex++] = topForm.frmTop;
                }
                return(arrFrm);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CloseAllNMDIForm:       Try to close all New MDI form
        /// </summary>
        /// <returns>
        /// true if succeed, false if not
        /// </returns>
        //
        //******************************************************************
        public virtual bool CloseAllNMDIForm() {
            bool    bRetVal = true;
		    
            if (m_arrTopForm != null) {
                foreach (TopForm topFormChild in m_arrTopForm) {
                    if (!((INMDIForm)topFormChild.frmTop).AskBeforeClosing()) {
                        bRetVal = false;
                        break;
                    }
                }
                if (bRetVal) {
                    while (m_arrTopForm.Count != 0) {
                        ((TopForm)m_arrTopForm[0]).frmTop.Close();
                    }
                }
            }
            return(bRetVal);		    
        }

        //******************************************************************
        //
        /// <summary>
        /// IsArrangeAvailable:         Returns true if the top form can be arranged
        ///                             using the specified method
        /// </summary>
        /// <param name="eArrangeStyle">Method to use to arrange the top form</param>
        /// <returns>
        /// false if no, true if yes
        /// </returns>
        //******************************************************************
        public virtual bool IsArrangeAvailable(ArrangeStyleE eArrangeStyle) {
            bool    bRetVal;

            if (m_arrTopForm != null) {
                if (m_iVisibleCount < 2) {
                    bRetVal = false;
                } else {
                    switch(eArrangeStyle) {
                    case ArrangeStyleE.Cascade:
                        bRetVal = true;
                        break;
                    case ArrangeStyleE.Horizontal:
                        bRetVal = (m_iVisibleCount == 2);
                        break;
                    default:
                        bRetVal = (m_iVisibleCount <= 8);
                        break;
                    }
                }
            } else {
                bRetVal = false;
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// ArrangeAllNMDIForm:         Arrange top form using the specified method
        /// </summary>
        /// <param name="eArrangeStyle">Method to use</param>
        //
        //******************************************************************
        public virtual void ArrangeAllNMDIForm(ArrangeStyleE eArrangeStyle) {
            int         iCount;
            int         iDeltaX;
            int         iDeltaY;
            int         iTop;
            int         iLeft;
            int         iWidth;
            int         iHeight;
            Rectangle   rectFormPos;
            Form        frm;

            if (IsArrangeAvailable(eArrangeStyle)) {
                rectFormPos = SystemInformation.WorkingArea;
                iTop        = rectFormPos.Top;
                iLeft       = rectFormPos.Left;
                iWidth      = rectFormPos.Width;
                iHeight     = rectFormPos.Height;
                iCount      = m_arrTopForm.Count;
                switch(eArrangeStyle) {
                case ArrangeStyleE.Horizontal:
                    iWidth      /= iCount;
                    iDeltaX      = iWidth;
                    iDeltaY      = 0;
                    break;                
                case ArrangeStyleE.Vertical:
                    iHeight     /= iCount;
                    iDeltaX      = 0;
                    iDeltaY      = iHeight;
                    break;
                default:
                    iDeltaX      = SystemInformation.HorizontalScrollBarThumbWidth;
                    iDeltaY      = SystemInformation.CaptionHeight;
                    iWidth      -= (iCount - 1) * iDeltaX;
                    iHeight     -= (iCount - 1) * iDeltaY;
                    break;
                }
                foreach (TopForm topForm in m_arrTopForm) {
                    if (topForm.bVisible) {
                        frm = topForm.frmTop;
                        if (frm.WindowState != FormWindowState.Normal) {
                            frm.WindowState = FormWindowState.Normal;
                        }
                        frm.SetDesktopBounds(iLeft, iTop, iWidth, iHeight);
                        frm.BringToFront();
                        iLeft += iDeltaX;
                        iTop  += iDeltaY;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// MinimizeAllNMDIForm:        Mimimize all top form
        /// </summary>
        //
        //******************************************************************
        public virtual void MinimizeAllNMDIForm() {
            Form    frm;
            
            foreach (TopForm topForm in m_arrTopForm) {
                if (topForm.bVisible) {
                    frm = topForm.frmTop;
                    if (frm.WindowState != FormWindowState.Minimized) {
                        frm.WindowState = FormWindowState.Minimized;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// RestoreAllNMDIForm:         Restore all top form
        /// </summary>
        //
        //******************************************************************
        public virtual void RestoreAllNMDIForm() {
            Form    frm;
            
            foreach (TopForm topForm in m_arrTopForm) {
                if (topForm.bVisible) {
                    frm = topForm.frmTop;
                    if (frm.WindowState != FormWindowState.Normal) {
                        frm.WindowState = FormWindowState.Normal;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// MaximizeAllNMDIForm:        Maximize all top form
        /// </summary>
        //
        //******************************************************************
        public virtual void MaximizeAllNMDIForm() {
            Form    frm;
            
            foreach (TopForm topForm in m_arrTopForm) {
                if (topForm.bVisible) {
                    frm = topForm.frmTop;
                    if (frm.WindowState != FormWindowState.Maximized) {
                        frm.WindowState = FormWindowState.Maximized;
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsMinimizeAllClick:          Minimize command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsMinimizeAllClick() {
            MinimizeAllNMDIForm();
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsRestoreAllClick:           Restore command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsRestoreAllClick() {
            RestoreAllNMDIForm();
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsMaximizeAllClick:          Maximize command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsMaximizeAllClick() {
            MaximizeAllNMDIForm();
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsArrangeVerticallyClick:   Arrange vertically command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsArrangeVerticallyClick() {
            ArrangeAllNMDIForm(ArrangeStyleE.Vertical);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsArrangeHorizontallyClick:  Arrange horizontally command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsArrangeHorizontallyClick() {
            ArrangeAllNMDIForm(ArrangeStyleE.Horizontal);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsArrangeInCascadeClick:     Cascade command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsArrangeInCascadeClick() {
            ArrangeAllNMDIForm(ArrangeStyleE.Cascade);
        }

        //******************************************************************
        //
        /// <summary>
        /// OnWindowsCloseAllClick:             Close all command
        /// </summary>
        //
        //******************************************************************
        private void OnWindowsCloseAllClick() {
            CloseAllNMDIForm();
        }
    } /* NMDIFormHelperBase */
} // Namespace
