using System;
using System.Windows.Forms;
using System.Xml;

namespace RepPP.Toolkit.Window {

    //*********************************************************	    
    //
    /// <summary>
    /// Implement the base class of a tree node attached to a TreeViewCInst 
    /// </summary>
    // 
    //*********************************************************	    
    public class TreeNodeBase : TreeNode, IUIConfiguration, System.IDisposable {
        /// <summary>true if child node has been read. false if not</summary>
        private bool                                    m_bChildrenRead;
        /// <summary>Contains the tooltip text used for this node.</summary>
        private string                                  m_strToolTipInfo;
        /// <summary>Node factory used to create child. If null, use this class ctor</summary>
        private TreeNodeGInst.IGInstChildNodeFactory    m_childNodeFactory;
        /// <summary>Type of the node</summary>
        private int                                     m_iNodeType;

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="strText">              Text to associate to the node</param>
        /// <param name="iNodeType">            Type to associate to the node</param>
        /// <param name="iImageIndex">          Index of the image for the unselected node</param>
        /// <param name="iSelectedImageIndex">  Index of the image for the selected node</param>
        /// <param name="childNodeFactory">     Class factory for child nodes attached to a group instance</param>
        //
        //*********************************************************	    
        protected TreeNodeBase(string                                    strText,
                               int                                       iNodeType,
                               int                                       iImageIndex,
                               int                                       iSelectedImageIndex,
                               TreeNodeGInst.IGInstChildNodeFactory      childNodeFactory) : base(strText, iImageIndex, iSelectedImageIndex)
        {
            m_strToolTipInfo    = "";
            m_childNodeFactory  = childNodeFactory;
            m_bChildrenRead     = false;
            m_iNodeType         = iNodeType;
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        //
        //*********************************************************	    
        protected TreeNodeBase() : this("", 0, 0, 0, null) {
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="childNodeFactory"> Class factory for child nodes attached to a group instance</param>
        //
        //*********************************************************	    
        protected TreeNodeBase(TreeNodeGInst.IGInstChildNodeFactory childNodeFactory) : this("", 0, -1, -1, childNodeFactory)
        {
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Init:                               Initialize the node
        /// </summary>
        /// <param name="treeFather">           Father treeview</param>
        /// <param name="bChild">               true if it's possible that child exist. False if impossible</param>
        //
        //*********************************************************	    
        protected virtual void Init(TreeViewCInst  treeFather,
                                    bool           bChild)
        {
            m_bChildrenRead = false;
            if (bChild) {
                Nodes.Add(new TreeNode("Dummy", 0, 0));
            } else {
                BaseReadChildren(treeFather);
            }
            SetNodeInfo(treeFather);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                Free the node
        /// </summary>
        //
        //*********************************************************	    
        public virtual void Dispose()
        {
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnBeforeChildRead:          Trigger the event before the child nodes are read
        /// </summary>
        /// <param name="treeFather">   Father treeview</param>
        /// <param name="e">            Event argument</param>        
        //
        //*********************************************************	    
        private void OnBeforeChildRead(TreeViewCInst treeFather, TreeViewCInst.BeforeChildReadEventArgs e)
        {
            if (treeFather != null) {
                treeFather.OnBeforeChildRead(e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnBeforeChildRead:      Trigger the event before the child nodes are read
        /// </summary>
        /// <param name="e">        Event argument</param>        
        //
        //*********************************************************	    
        protected virtual void OnBeforeChildRead(TreeViewCInst.BeforeChildReadEventArgs e)
        {
            OnBeforeChildRead(TreeView as TreeViewCInst, e);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnAfterChildRead:           Trigger the event after the child nodes are read
        /// </summary>
        /// <param name="treeFather">   Father treeview</param>
        /// <param name="e">            Event argument</param>        
        //
        //*********************************************************	    
        private void OnAfterChildRead(TreeViewCInst treeFather, TreeViewCInst.AfterChildReadEventArgs e)
        {
            if (treeFather != null) {
                treeFather.OnAfterChildRead(e);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// OnAfterChildRead:       Trigger the event after the child nodes are read
        /// </summary>
        /// <param name="e">        Event argument</param>        
        //
        //*********************************************************	    
        protected virtual void OnAfterChildRead(TreeViewCInst.AfterChildReadEventArgs e)
        {
            OnAfterChildRead(TreeView as TreeViewCInst, e);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ChildNodeFactory:   The factory used to creates the child nodes
        /// </summary>
        //
        //*********************************************************	    
        public TreeNodeGInst.IGInstChildNodeFactory ChildNodeFactory {
            get {
                return(m_childNodeFactory);
            }
            set {
                m_childNodeFactory = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Tooltip:                Treenode tooltip
        /// </summary>
        //
        //*********************************************************	    
        public virtual string Tooltip {
            get {
                return(m_strToolTipInfo);
            }
            set {
                m_strToolTipInfo = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// BaseReadChildren:       Read the children
        /// </summary>
        //
        //*********************************************************	    
        private void BaseReadChildren(TreeViewCInst treeFather)
        {
            TreeViewCInst.AfterChildReadEventArgs     afterArg;
            TreeViewCInst.BeforeChildReadEventArgs    beforeArg;
            
            if (!m_bChildrenRead) {
                Nodes.Clear();
                beforeArg       = new TreeViewCInst.BeforeChildReadEventArgs(this);
                afterArg        = new TreeViewCInst.AfterChildReadEventArgs(this);
                m_bChildrenRead = true;
                OnBeforeChildRead(treeFather, beforeArg);
                OnAfterChildRead(treeFather, afterArg);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ReadChildren:           Read the children
        /// </summary>
        //
        //*********************************************************	    
        public virtual void ReadChildren()
        {
            BaseReadChildren(TreeView as TreeViewCInst);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// IsChildrenRead:         Check if the children has been read
        /// </summary>
        //
        //*********************************************************	    
        public bool IsChildrenRead
        {
            get {
                return(m_bChildrenRead);
            }
            set {
                m_bChildrenRead = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// IsRecursive:            true if the node is recursive, no if not.
        /// </summary>
        //
        //*********************************************************	    
        public virtual bool IsRecursive
        {
            get {
                return(false);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SetNodeInfo:                Set the node information
        /// </summary>
        /// <param name="treeFather">   Father treeview</param>
        //
        //*********************************************************	    
        protected virtual void SetNodeInfo(TreeViewCInst treeFather)
        {
            if (treeFather != null) {
                treeFather.OnSetNodeInfo(new TreeViewCInst.SetNodeInfoEventArgs(this));
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SetNodeInfo:    Set the node information
        /// </summary>
        //
        //*********************************************************	    
        public void SetNodeInfo()
        {
            SetNodeInfo(TreeView as TreeViewCInst);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// IsNodeAncestorOf:   Checks if the node is ancestor of the specified one
        /// </summary>
        //
        //*********************************************************	    
        public bool IsNodeAncestorOf(TreeNode node) {
            if (node != null) {
                node = node.Parent;
                while (node != null && node != this) {
                    node = node.Parent;
                }
            }
            return(node == this);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// Clone:                      Clone the node and all its descendant
        /// </summary>
        /// <returns>
        /// Node key
        ///</returns>
        //
        //*********************************************************	    
        public override object Clone() {
            TreeNodeBase    nodeRetVal;

            nodeRetVal = base.Clone() as TreeNodeBase;
            nodeRetVal.m_bChildrenRead      = m_bChildrenRead;
            nodeRetVal.m_strToolTipInfo     = m_strToolTipInfo;
            nodeRetVal.m_childNodeFactory   = m_childNodeFactory;
            nodeRetVal.m_iNodeType          = m_iNodeType;
            return(nodeRetVal);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetNodeKey:                 Gets the key of the node
        /// </summary>
        /// <returns>
        /// Node key
        ///</returns>
        //
        //*********************************************************	    
        public virtual string GetNodeKey()
        {
            return("BASE:" + Text);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// NodeType:                   Type of the node. Used to select the related editor.
        /// </summary>
        //
        //*********************************************************	    
        public virtual int NodeType
        {
            get {
                return(m_iNodeType);
            }
            set {
                m_iNodeType = value;
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// SelectAttachedObject:       Select the object attached to the node
        /// </summary>
        //
        //*********************************************************	    
        public virtual void SelectAttachedObject()
        {
            TreeNodeBase    itemFather;
            
            if (Parent != null && Parent is TreeNodeBase) {
                itemFather = (TreeNodeBase)Parent;
                itemFather.SelectAttachedObject();
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetSiblingCount:        Gets the number of sibling
        /// </summary>
        /// <returns>
        /// Number of nodes of the same type.
        /// </returns>
        //
        //*********************************************************	    
        public virtual int GetSiblingCount()
        {
            return(1);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GetSiblingPos:          Gets the position in the sibling
        /// </summary>
        /// <returns>
        /// Position
        /// </returns>
        //
        //*********************************************************	    
        public virtual int GetSiblingPos()
        {
            return(0);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindSiblingNode:        Find a sibling node using a position
        /// </summary>
        /// <param name="iPos">     Position (0..n)</param>
        /// <returns>
        /// null or the specified Node
        /// </returns>
        //
        //*********************************************************	    
        public virtual TreeNodeBase FindSiblingNode(int iPos)
        {
            return((iPos == 0) ? this : null);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// GroupInstance:      Group instance attached to this node
        /// </summary>
        //
        //*********************************************************	    
        public virtual RepPP.GroupInstance GroupInstance {
            get {
                return(null);
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindChildGroupInstance:     Find a child group instance if any
        /// </summary>
        /// <param name="grp">          Group</param>
        /// <returns>
        /// null or the specified group instance
        /// </returns>
        //
        //*********************************************************	    
        public virtual RepPP.GroupInstance FindChildGroupInstance(RepPP.Group grp)
        {
            return(null);
        }

        //*********************************************************	    
        //
        /// <summary>
        /// FindNodeUsingKey:       Finds a node using a key in the specified node collection
        /// </summary>
        //
        //*********************************************************	    
        public static TreeNodeBase FindNodeUsingKey(TreeNodeCollection nodes, string strKey)
        {
            TreeNodeBase    nodeRetVal = null;
            TreeNodeBase    nodeBase;
            
            foreach (TreeNode node in nodes) {
                nodeBase = node as TreeNodeBase;
                if (nodeBase != null) {
                    if (nodeBase.GetNodeKey() == strKey) {
                        nodeRetVal = nodeBase;
                        break;
                    }
                }
            }
            return(nodeRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadState               Load the UI configuration state
        /// </summary>
        /// <param name="helper">   Helper object</param>
        /// <param name="node">     Node containing the serialization</param>
        //
        //******************************************************************
        public virtual void LoadState(UIConfigurationHelper helper, System.Xml.XmlNode node) {
        }

        //******************************************************************
        //
        /// <summary>
        /// SaveState               Save the UI configuration state
        /// </summary>
        /// <param name="helper">   Helper object</param>
        /// <param name="node">     Node containing the serialization</param>
        //
        //******************************************************************
        public virtual void SaveState(UIConfigurationHelper helper, System.Xml.XmlNode node) {
        }
    }
}
